@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 
 package Tie::RefHash::Weak;
-use base qw/Tie::RefHash/;
+use base qw/Tie::RefHash Exporter/;
 
 use strict;
 use warnings;
@@ -12,7 +12,9 @@ use overload ();
 
 use B qw/svref_2object CVf_CLONED/;
 
-our $VERSION = 0.07;
+our $VERSION = 0.08;
+our @EXPORT_OK = qw 'fieldhash fieldhashes';
+our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
 use Scalar::Util qw/weaken reftype/;
 use Variable::Magic qw/wizard cast getdata/;
@@ -50,29 +52,16 @@ sub STORE {
 
 		my $objects;
 
-		# blech, any idea how to clean this up?
-
-		if ( reftype $k eq 'SCALAR' ) {
-			$objects = getdata( $$k, $wiz )
-				or cast( $$k, $wiz, ( $objects = [] ) );
-		} elsif ( reftype $k eq 'HASH' ) {
-			$objects = getdata ( %$k, $wiz )
-				or cast( %$k, $wiz, ( $objects = [] ) );
-		} elsif ( reftype $k eq 'ARRAY' ) {
-			$objects = getdata ( @$k, $wiz )
-				or cast( @$k, $wiz, ( $objects = [] ) );
-		} elsif ( reftype $k eq 'GLOB' or reftype $k eq 'IO' ) {
-			$objects = getdata ( *$k, $wiz )
-				or cast( *$k, $wiz, ( $objects = [] ) );
-		} elsif ( reftype $k eq 'CODE' ) {
+		if ( reftype $k eq 'CODE' ) {
 			unless ( svref_2object($k)->CvFLAGS & CVf_CLONED ) {
 				warnings::warnif("Non closure code references never get garbage collected: $k");
 			} else {
-				$objects = getdata ( &$k, $wiz )
-					or cast( &$k, $wiz, ( $objects = [] ) );
+				$objects = &getdata ( $k, $wiz )
+					or &cast( $k, $wiz, ( $objects = [] ) );
 			}
 		} else {
-			die "patches welcome";
+			$objects = &getdata( $k, $wiz )
+				or &cast( $k, $wiz, ( $objects = [] ) );
 		}
 
 		unless ( grep { $_ == $s } @$objects ) {
@@ -89,6 +78,16 @@ sub STORE {
 	$v;
 }
 
+sub fieldhash(\%) {
+	tie %{$_[0]}, __PACKAGE__;
+	return $_[0];
+}
+
+sub fieldhashes {
+	tie %{$_}, __PACKAGE__ for @_;
+	return @_;
+}
+
 __PACKAGE__
 
 __END__
@@ -102,9 +101,13 @@ Tie::RefHash::Weak - A Tie::RefHash subclass with weakened references in the key
 =head1 SYNOPSIS
 
 	use Tie::RefHash::Weak;
-
 	tie my %h, 'Tie::RefHash::Weak';
 
+	# OR:
+
+	use Tie::RefHash::Weak 'fieldhash';
+	fieldhash my %h;
+
 	{ # new scope
 		my $val = "foo";
 
@@ -133,6 +136,26 @@ This subclass of L<Tie::RefHash> has weak keys, instead of strong ones. The
 values are left unaltered, and you'll have to make sure there are no strong
 references there yourself.
 
+=head1 FUNCTIONS
+
+For compatibility with L<Hash::Util::FieldHash>, this module will, upon
+request, export the following two functions. You may also write
+C<use Tie::RefHash::Weak ':all'>.
+
+=over 4
+
+=item fieldhash %hash
+
+This ties the hash and returns a reference to it.
+
+=item fieldhashes \%hash1, \%hash2 ...
+
+This ties each hash that is passed to it as a reference. It returns the
+list of references in list context, or the number of hashes in scalar
+context.
+
+=back
+
 =head1 THREAD SAFETY
 
 L<Tie::RefHash> version 1.32 and above have correct handling of threads (with
@@ -144,6 +167,21 @@ Version 0.02 and later of Tie::RefHash::Weak depend on a thread-safe version of
 Tie::RefHash anyway, so if you are using the latest version this should already
 be taken care of for you.
 
+=head1 5.10.0 COMPATIBILITY
+
+Due to a minor change in Perl 5.10.0 a bug in the handling of magic freeing was
+uncovered causing segmentation faults.
+
+This has been patched but not released yet, as of 0.08.
+
+=head1 CAVEAT
+
+You can use an LVALUE reference (such as C<\substr ...>) as a hash key, but
+due to a bug in perl (see
+L<http://rt.perl.org/rt3/Public/Bug/Display.html?id=46943>) it might not be 
+possible to weaken a reference to it, in which case the hash element will 
+never be deleted automatically.
+
 =head1 AUTHORS
 
 Yuval Kogman <nothingmuch@woobling.org>
